/*
 * Copyright 2015-2017, Thomas Maier-Komor
 * LICENSE: GPLv3, see file LICENSE for details
 */

#include <assert.h>
#include <errno.h>
#ifndef __CYGWIN__
#include <execinfo.h>
#endif
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "config.h"
#include "log.h"


const char *Copyright = "Copyright 2015-2019, Thomas Maier-Komor";
const char *License = "GPLv3 - see file LICENSE for details";
const char *Version = VERSION;
const char *McastAddr = MCAST_ADDR;
unsigned Port = MCAST_PORT;

int (*Endfunc)() = 0;


void sigSegVhandler(int sig)
{
	assert((sig == SIGSEGV) || (sig == SIGABRT));
#ifdef __linux
	void *bt[128];
	int n = backtrace(bt,sizeof(bt)/sizeof(void*));
	char buf[4096];
	char *a;
	if (sig == SIGABRT)
		a = stpcpy(buf,"SIGABRT occured at:\n");
	else
		a = stpcpy(buf,"SIGSEGV occured at:\n");
	(void)write(Log != -1 ? Log : STDERR_FILENO,buf,a-buf);
	if (Endfunc)
		Endfunc();
	backtrace_symbols_fd(bt,n,Log != -1 ? Log : STDERR_FILENO);
#endif
	unlink(LOCKFILE);
	signal(SIGSEGV,SIG_DFL);
	kill(getpid(),SIGSEGV);
}


void print_version()
{
	printf("HostScope Version %s\n%s\nLicense: %s\n\n"
		"This program is free software: you can redistribute it and/or modify\n"
		"it under the terms of the GNU General Public License as published by\n"
		"the Free Software Foundation, either version 3 of the License, or\n"
		"(at your option) any later version.\n"
		"\n"
		"This program is distributed in the hope that it will be useful,\n"
		"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
		"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
		"GNU General Public License for more details.\n"
		"\n"
		"You should have received a copy of the GNU General Public License\n"
		"along with this program.  If not, see <http://www.gnu.org/licenses/>. \n"
		, Version, Copyright, License); 
}


void kill_daemon()
{
	FILE *pidf = fopen(LOCKFILE,"r");
	if (0 == pidf)
		fatal("lock file " LOCKFILE " does not exist - no hostscoped running?\n");
	long pid;
	if (fscanf(pidf,"%ld",&pid) != 1)
		fatal("lock file " LOCKFILE " seems to be corrupt - pid not found\n");
	fclose(pidf);
	if (-1 == kill(pid,SIGTERM)) {
		if (errno == ESRCH) {
			if (-1 == unlink(LOCKFILE))
				warning("unable to remove lock file " LOCKFILE ": %s\n",strerror(errno));
		}
		fatal("unable to kill daemon with pid %d: %s\n",pid,strerror(errno));
	}
	if (-1 == unlink(LOCKFILE))
		warning("unable to remove lock file " LOCKFILE ": %s\n",strerror(errno));
	exit(EXIT_SUCCESS);
}


void setMulticastPort(char *a)
{
	long p = strtol(a,0,0);
	if (p &~0xffff)
		fatal("unable to set port to %d: value out of range\n",p);
	Port = p;
	info("multicast group port set to %u\n",Port);
}


void setMulticastAddr(char *a)
{
	char *c = strrchr(a,':');
	if (c) {
		*c = 0;
		setMulticastPort(++c);
		McastAddr = a;
		info("setting multicast group address to %s, port %u\n",a,Port);
	} else {
		McastAddr = a;
		info("setting multicast group address to %s\n",a);
	}
}


