/*
 * Copyright 2015-2017, Thomas Maier-Komor
 * LICENSE: GPLv3, see file LICENSE for details
 */

#include <errno.h>
#include <fcntl.h>
#include <inttypes.h>
#include <string.h>
#include <unistd.h>

#include <vector>

#ifdef WFC
#include "hostscope_wfc.h"
#else
#include "hostscope.pb.h"
#endif

#include "log.h"
#include "support.h"

using namespace std;

struct NetDev
{
	char devname[16];
	uint64_t ts;
	uint64_t rb,rp,re,rd,tb,tp,te,td;

	NetDev(const char *dn,uint64_t _ts,uint64_t _rb,uint64_t _rp,uint64_t _re,uint64_t _rd,uint64_t _tb,uint64_t _tp,uint64_t _te,uint64_t _td)
	: devname()
	, ts(_ts)
	, rb(_rb),rp(_rp),re(_re),rd(_rd),tb(_tb),tp(_tp),te(_te),td(_td)
	{ strcpy(devname,dn); }
};

vector<NetDev> NetDevs;


void read_networks(HostScope *t)
{
	TimeStat(__FUNCTION__);
	uint64_t ts = get_us_time();
	static int nd = open("/proc/net/dev",O_RDONLY);
	if (nd == -1) {
		warnonce("unable to open /proc/net/dev: %s\n",strerror(errno));
		return;
	}
	char buf[4096];
	int n = pread(nd,buf,sizeof(buf),0);
	if (n < 0) {
		warnonce("unable to read /proc/net/dev: %s\n",strerror(errno));
		return;
	}
	assert(n < (int)sizeof(buf));
	buf[n] = 0;
	char *at = strchr(buf,'\n');
	assert(at);
	++at;
	while (at && *at) {
		at = strchr(at,'\n');
		if (at == 0)
			return;
		++at;
		if (*at == 0)
			return;
		char *devname = at;
		at = strchr(at,':');
		assert(at);
		while (*devname == ' ')
			++devname;
		*at = 0;
		++at;
		uint64_t rb,rp,re,rd,tb,tp,te,td;
		int f = sscanf(at,"%" SCNu64 " %" SCNu64 " %" SCNu64 " %" SCNu64 " %*u %*u %*u %*u %" SCNu64 " %" SCNu64 " %" SCNu64 " %" SCNu64 " %*u %*u %*u %*u"
				, &rb, &rp, &re, &rd
				, &tb, &tp, &te, &td);
		if (f == 8) {
			size_t i = 0;
			size_t ns = NetDevs.size();
			while ((i < ns) && strcmp(devname,NetDevs[i].devname))
				++i;
			if (i == ns)  {
				NetDevs.push_back(NetDev(devname,ts,rb,rp,re,rd,tb,tp,te,td));
				continue;
			}
			NetDev &d = NetDevs[i];
			double dt = (double)((ts - d.ts)*1E-6);
			HostScope_NetDevice *n = 0;
			while (i >= (size_t)t->netdevices_size())
				t->add_netdevices();
			n = t->mutable_netdevices(i);
			if (n->devname().empty())
				n->set_devname(d.devname);
			//dbug("%s %u %u %u %u %f\n",d.devname,rp-d.rp,tp-d.tp,rb-d.rb,tb-d.tb,dt);
			n->set_nreceived(dbl2fxptE4((double)(rp-d.rp)/dt));
			n->set_kreceived(dbl2fxptE4((double)(rb-d.rb)/1024.0/dt));
			n->set_ntransmit(dbl2fxptE4((double)(tp-d.tp)/dt));
			n->set_ktransmit(dbl2fxptE4((double)(tb-d.tb)/1024.0/dt));
			n->set_dreceived(dbl2fxptE4((double)(rd-d.rd)/dt));
			n->set_dtransmit(dbl2fxptE4((double)(td-d.td)/dt));
			n->set_ereceived(re);
			n->set_etransmit(te);
			d.ts = ts;
			d.rp = rp;
			d.rb = rb;
			d.re = re;
			d.rd = rd;
			d.tp = tp;
			d.tb = tb;
			d.te = te;
			d.td = td;
		}
	}
}


